-----------------------------------------------------------------------------------------------
-- Client Lua Script for KuronaPets
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- KuronaPets Module Definition
-----------------------------------------------------------------------------------------------
local KuronaPets = {} 
local Version = 1.3
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 local Phrases = { "I am Revived!", "Am I your favorite?", "Gotta catch them all!","Yes?","I was a bit busy, but whatever.","En garde!","Winter is coming","I'm here","...","::blush::","It's dangerous to go alone. Take me","<3 Kurona","Notice me Senpai","^_^","^.^","More Cowbell","Say hello to my little friend","Go ahead, make my day","Let's Go!","I see dead people","Allons-y!","Giggity","Onward!","Sup?","KuronaPets",
 "Adventure!","::whisper::","!","?","I have no memory of this place","?","!?","Alright!","Let's Go","Shiny!","ma ma ma my Kurona","Sweet!","crabine pls","carbino pls","Hi","rawr","Poof","Go Go Go","so sleepy","<3 Time Travel","What time is it?","Is this Heaven?","Oh, it's just you","You're kinda poor","Thanks Obama","Beep Beep Richie!","Finally","Nice","Awesome"
 }
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function KuronaPets:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function KuronaPets:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- KuronaPets OnLoad
-----------------------------------------------------------------------------------------------
function KuronaPets:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("KuronaPets.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("KPSprite.xml","KPSprite")
	
end

-----------------------------------------------------------------------------------------------
-- KuronaPets OnDocLoaded
-----------------------------------------------------------------------------------------------
function KuronaPets:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "KuronaPetsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.tKnownPets = {}
		self.tUnknownPets = {}
	
		self.bShowUnknown = false

		
		self.PetFlyout = Apollo.LoadForm(self.xmlDoc, "PetFlyout", "FixedHudStratumLow", self)
		self.PetFlyoutFrame = Apollo.LoadForm(self.xmlDoc, "PetPopupFrame", nil, self)
		if self.PetFlyoutFrame ~= nil then
			self.PetFlyout:FindChild("PetPopoutBtn"):SetCheck(false)
			self.PetFlyout:FindChild("PetPopoutBtn"):AttachWindow(self.PetFlyoutFrame)
		end
		
		--defaults
		if self.bAutoSummon == nil then
			self.bAutoSummon = false
		end
		if self.bNoRaid == nil then
			self.bNoRaid = false
		end
		if self.bCanTalk == nil then
			self.bCanTalk = true
		end
		if self.nSpeakChance == nil then
			self.nSpeakChance = 70
		end
		if self.tPos then
			self.PetFlyout:SetAnchorOffsets(unpack(self.tPos))
		end
		self:MoveFlyout()	
		
		--Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
		Apollo.RegisterTimerHandler("RedrawTimer", "RedrawPets", self)
		Apollo.RegisterTimerHandler("AutoSummonTimer", "TryToAutoSummon", self)
		Apollo.RegisterTimerHandler("TalkTimer", "TryToSpeak", self)
		Apollo.RegisterEventHandler("GenericEvent_CollectablesReady", "OnReady", self)
		Apollo.RegisterEventHandler("AbilityBookChange", "RedrawPets", self)
		Apollo.RegisterSlashCommand("kpets", "OnConsole", self)
		Apollo.RegisterEventHandler("CombatLogResurrect", "OnRevive", self)
		Apollo.RegisterEventHandler("CombatMomentum", "OnCombatMomentum", self)
		
		self.SummonTries = 0
		
		self:FillOptions()
		
	    self.wndMain:Show(false, true)
	end
	
--	if GameLib.GetPlayerUnit() ~= nil then
--		self:OnCharacterCreated()
--	end

end

-----------------------------------------------------------------------------------------------
-- KuronaPets Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here


-----------------------------------------------------------------------------------------------
-- KuronaPetsForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function KuronaPets:OnOK()
	self.wndMain:Close() -- hide the window
	self.PetFlyout:SetStyle("RequireMetaKeyToMove",false)
	self.PetFlyout:FindChild("PetPopoutBtn"):Enable(true)
	self.PetFlyout:SetStyle("Picture",false)	
	self.PetFlyout:SetStyle("IgnoreMouse",true)	
end

-- when the Cancel button is clicked
function KuronaPets:OnCancel()
	self:OnOK() -- hide the window
end

function KuronaPets:PopulatePetList()

end


---------------------------------------------------------------------------------------------------
-- PetBtn Functions
---------------------------------------------------------------------------------------------------

function KuronaPets:OnGenerateTooltip( wndHandler, wndControl, eToolTipType, x, y )
	wndControl:SetTooltip(wndHandler:GetData().strName)
end

-----------------------------------------------------------------------------------------------
-- KuronaPets Instance
-----------------------------------------------------------------------------------------------
local KuronaPetsInst = KuronaPets:new()
KuronaPetsInst:Init()




function KuronaPets:RedrawPets()
	local wndPetList = self.wndMain:FindChild("PetList")
	local arPetList = CollectiblesLib.GetVanityPetList()
	local PopoutList = self.PetFlyoutFrame:FindChild("PetPopoutList")
	PopoutList:DestroyChildren()
	table.sort(arPetList, function(a,b) return (a.bIsKnown and not b.bIsKnown) or (a.bIsKnown == b.bIsKnown and a.strName < b.strName) end)
	
	for idx = 1, #arPetList do
		local tPetInfo = arPetList[idx]
		local wndPet = nil
		
		if tPetInfo.bIsKnown then		
			wndPet = Apollo.LoadForm(self.xmlDoc, "PetBtn", PopoutList, self)
			local wndActionButton = wndPet:FindChild("PetBtnIcon") --:FindChild("ActionBarButton")
			wndActionButton:SetData(tPetInfo.nId)
			wndActionButton:SetSprite(tPetInfo.splObject and tPetInfo.splObject:GetIcon() or "Icon_ItemArmorWaist_Unidentified_Buckle_0001")
			wndPet:SetData(tPetInfo)
			wndPet:SetTooltip(tPetInfo.strName)

			if self.nSelectedPet == nil then
				self.nSelectedPet = tPetInfo
			end
			if self.nSelectedPetId == tPetInfo.nId then
				self.nSelectedPet = tPetInfo
				self.nSelectedPetId = nil
			end		
		end
	end	
	
	self:ArrangeList()
	
	if self.nSelectedPet then
		self:RedrawSelectedPet()
	end
end

function KuronaPets:OnReady()
	self:RedrawPets()
end

function KuronaPets:OnMountBtn(wndHandler, wndControl)
	self.nSelectedPet = wndControl:GetData()
	self.PetFlyoutFrame:Show(false)
	self:RedrawSelectedPet()
	self:OnSummonPet(nil,nil,0)
end

function KuronaPets:RedrawSelectedPet()
	self.PetFlyout:FindChild("Icon"):SetSprite(self.nSelectedPet.splObject and self.nSelectedPet.splObject:GetIcon())
end

function KuronaPets:OnCharacterCreated()
	Apollo.CreateTimer("RedrawTimer", 2, false)
end

function KuronaPets:ArrangeList()
	local wndPetList = self.PetFlyoutFrame:FindChild("PetPopoutList")
	
	wndPetList:ArrangeChildrenVert()
	wndPetList:SetVScrollPos(0)
	--self.PetFlyoutFrame:RecalculateContentExtents()
end

function KuronaPets:OnSummonPet( wndHandler, wndControl, eMouseButton )
if self.nSelectedPet == nil then return end
	local arPets = GameLib.GetPlayerPets()
	local bPetSummoned = false
	if arPets then
		for idx, unitPet in pairs (arPets) do
			if unitPet:GetName() == self.nSelectedPet.strName then
				bPetSummoned = true
				break
			end
		end
	end

	if not bPetSummoned or eMouseButton== 1 then
		GameLib.SummonVanityPet(self.nSelectedPet.nId)
		if self.bCanTalk then
			Apollo.CreateTimer("TalkTimer", 3, false)
		end
	end
end


function KuronaPets:OnSave(eLevel,backup)	
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tsave = {}
	tsave.nSelectedPetId = self.nSelectedPet.nId
	tsave.bAutoSummon = self.bAutoSummon
	tsave.bNoRaid = self.bNoRaid
	tsave.bCanTalk = self.bCanTalk
	tsave.nSpeakChance = self.nSpeakChance
	tsave.tPos = {self.PetFlyout:GetAnchorOffsets()}
	return tsave
end

function KuronaPets:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	self.nSelectedPetId = saveData.nSelectedPetId
	self.bAutoSummon = saveData.bAutoSummon
	self.bNoRaid = saveData.bNoRaid
	self.bCanTalk = saveData.bCanTalk
	self.nSpeakChance = saveData.nSpeakChance
	self.tPos = saveData.tPos
	end

function KuronaPets:OnRevive(unit)
	if self.bNoRaid and GroupLib.InRaid() then return end
	
	if self.bAutoSummon then
		Apollo.CreateTimer("AutoSummonTimer", 0.5, false)
	end
end

function KuronaPets:FillOptions()
	self.wndMain:FindChild("bAutoSummon"):SetCheck(self.bAutoSummon)
	self.wndMain:FindChild("bNoRaid"):SetCheck(self.bNoRaid)
	self.wndMain:FindChild("bCanTalk"):SetCheck(self.bCanTalk)
	--Slider
	self.wndMain:FindChild("nSpeakChance"):SetValue(self.nSpeakChance)
	self.wndMain:FindChild("nSpeakChanceValue"):SetText((self.nSpeakChance) .."%")	
end

function KuronaPets:OnConsole()
	self.wndMain:Invoke()
	self.PetFlyout:SetStyle("RequireMetaKeyToMove",false)
	self.PetFlyout:SetStyle("Picture",true)
	self.PetFlyout:SetStyle("IgnoreMouse",false)
	self.PetFlyoutFrame:Show(false)
	self.PetFlyout:FindChild("PetPopoutBtn"):Enable(false)
end

function KuronaPets:OnSettingsChange( wndHandler, wndControl, eMouseButton )
	self[wndHandler:GetName()] = wndHandler:IsChecked()
end

function KuronaPets:TryToAutoSummon()
	self:OnSummonPet(nil,nil,0)
	self.SummonTries = self.SummonTries + 1
	
	local arPets = GameLib.GetPlayerPets()
	local bPetSummoned = false
	if arPets then
		for idx, unitPet in pairs (arPets) do
			if unitPet:GetName() == self.nSelectedPet.strName then
				bPetSummoned = true
				break
			end
		end
	end
	
	if not bPetSummoned and self.SummonTries < 20 then
		self.PetFlyout:FindChild("SummonFlash"):Show(true,false)
		Apollo.CreateTimer("AutoSummonTimer", 0.5, false)
	else
		Apollo.StopTimer("AutoSummonTimer")
		self.SummonTries = 0
		self.PetFlyout:FindChild("SummonFlash"):Show(false,false)
	end
end

function KuronaPets:OnMouseOver( wndHandler, wndControl, x, y )
--	wndHandler:FindChild("Icon"):SetBGColor("ffffff00")
--	Print("over")
end

function KuronaPets:OnMouseExit( wndHandler, wndControl, x, y )
--	wndHandler:FindChild("Icon"):SetBGColor("ffffffff")
--	Print("exit")
end


function KuronaPets:TryToSpeak(strMsg)
	local arPets = GameLib.GetPlayerPets()
	local bPetSummoned = false
	if arPets then
		for idx, unitPet in pairs (arPets) do
			if unitPet:GetName() == self.nSelectedPet.strName then
				self.myPet = unitPet
				bPetSummoned = true
				break
			end
		end
	end
	if bPetSummoned then
		if strMsg then
			self.myPet:AddTextBubble(strMsg)
		elseif math.random(100) <= self.nSpeakChance  then
			self.myPet:AddTextBubble(self:GetRandomSaying())
		end
	end
end

function KuronaPets:GetRandomSaying()
	local strText = Phrases[math.random(#Phrases)]
	return strText
end

function KuronaPets:OnCombatMomentum( eMomentumType, nCount, strText )
if self.bCanTalk then
	-- Passes: type enum, player's total count for that bonus type, string combines these things (ie. "3 Evade")
	--if eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 2 then
	--	self:TryToSpeak(Apollo.GetString("FloatText_DoubleKill"))
	if eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 3 then
		self:TryToSpeak(Apollo.GetString("FloatText_TripleKill"))
	elseif eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 5 then
		self:TryToSpeak(Apollo.GetString("FloatText_PentaKill"))
	elseif eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 8 then
		self:TryToSpeak("Uber Kill")
	elseif eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 12 then
		self:TryToSpeak("Epic Kill")
	elseif eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 16 then
		self:TryToSpeak("Mega Kill")
	elseif eMomentumType == CombatFloater.CodeEnumCombatMomentum.KillChain and nCount == 20 then
		self:TryToSpeak("Godly Kill")
	end
end

end


function KuronaPets:OnSliderChanged( wndHandler, wndControl, fNewValue, fOldValue, bOkToChange )
	local slider = wndHandler:GetName()
	self[slider] = wndHandler:GetValue() 
	wndHandler:GetParent():GetParent():SetText(self[slider] .. "%")
end

function KuronaPets:ResetPos( wndHandler, wndControl, eMouseButton )
	self.PetFlyout:SetAnchorOffsets(-117,-113,4,-32)
	self:MoveFlyout()
end

function KuronaPets:MoveFlyout()
	self.tPos = {self.PetFlyout:GetAnchorOffsets()}
	local l,t,r,b = self.tPos[1],self.tPos[2],self.tPos[3],self.tPos[4]
	self.PetFlyoutFrame:SetAnchorOffsets(l-6,t-448,r+6,b-76)
end
